#!/bin/sh

#
# Script for creating the TPC-R database
#
# written 2001-08-12 by Tord Hammer <thammer@hammer-software.de>
#

usage()
{
    echo "Usage: maketpc.sh [-h] [-s X.XX] [-p /path] [-q] [-b XXXX]"
    echo "       -h ... shows usage"
    echo "       -s ... scales the database (default: 1.00)"
    echo "              a scale-factor of 1.00 results in about 1 GB of data or"
    echo "              about 1,4 GB of gdb-file"
    echo "              note: since the database isnt split across multiple files,"
    echo "              you should not try a large scale without 64bit i/o-support"
    echo "       -p ... prefix for the database (default: /tmp)"
    echo "       -b ... set cache to XXXX buffers (default: 10000)"
    echo "       -q ... quiet backup and restore"
    exit 1
}

SCALE=1.00
PREFIX=/tmp
QUIET=N
BUFFERS=10000

while [ -n "$1" ]; do
	if [ "$1" = "-h" ]; then 
		usage; 
	elif [ "$1" = "-s" ]; then
		shift
		if [ -z "$1" ]; then usage; fi
		SCALE="$1"
	elif [ "$1" = "-p" ]; then
		shift
		if [ -z "$1" ]; then usage; fi
		PREFIX="$1"
	elif [ "$1" = "-q" ]; then
		QUIET=Y
	elif [ "$1" = "-b" ]; then
		shift
		if [ -z "$1" ]; then usage; fi
		BUFFERS="$1"
	else
		usage;
	fi
	shift
done

# create a working file
SCRIPT=`basename "$0"`
TMPSQL=`mktemp $PREFIX/$SCRIPT.XXXXXX`
if [ $? -ne 0 ]; then
	echo "can't create temp file, exiting"
	exit 1
fi

echo "Trying to drop $PREFIX/tpcr.gdb ..."
cat >$TMPSQL <<!
connect '$PREFIX/tpcr.gdb';
drop database;
!
isql -i $TMPSQL
if [ $? -ne 0 ]; then
	echo "failed."
	exit 1
fi

echo "Creating new $PREFIX/tpcr.gdb ..."
cat >$TMPSQL <<!
create database '$PREFIX/tpcr.gdb'
	page_size 4096;

create table PART (
	P_PARTKEY	integer not null,
	P_NAME		varchar(55),
	P_MFGR		char(25),
	P_BRAND		char(10),
	P_TYPE		varchar(25),
	P_SIZE		integer,
	P_CONTAINER	char(10),
	P_RETAILPRICE	numeric(18,2),
	P_COMMENT	varchar(23)
);

alter table PART
	add constraint PART_PK primary key (P_PARTKEY);

create table SUPPLIER (
	S_SUPPKEY	integer not null,
	S_NAME		char(25),
	S_ADDRESS	varchar(40),
	S_NATIONKEY	integer not null,
	S_PHONE		char(15),
	S_ACCTBAL	numeric(18,2),
	S_COMMENT	varchar(101)
);

alter table SUPPLIER
	add constraint SUPPLIER_PK primary key (S_SUPPKEY);

create table PARTSUPP (
	PS_PARTKEY	integer not null,
	PS_SUPPKEY	integer not null,
	PS_AVAILQTY	integer,
	PS_SUPPLYCOST	numeric(18,2),
	PS_COMMENT	varchar(199)
);

alter table PARTSUPP
	add constraint PARTSUPP_PK primary key (PS_PARTKEY, PS_SUPPKEY);

create table CUSTOMER (
	C_CUSTKEY	integer not null,
	C_NAME		varchar(25),
	C_ADDRESS	varchar(40),
	C_NATIONKEY	integer not null,
	C_PHONE		char(15),
	C_ACCTBAL	numeric(18,2),
	C_MKTSEGMENT	char(10),
	C_COMMENT	varchar(117)
);

alter table CUSTOMER
	add constraint CUSTOMER_PK primary key (C_CUSTKEY);

create table ORDERS (
	O_ORDERKEY	integer not null,
	O_CUSTKEY	integer not null,
	O_ORDERSTATUS	char(1),
	O_TOTALPRICE	numeric(18,2),
	O_ORDERDATE	date,
	O_ORDERPRIORITY	char(15),
	O_CLERK		char(15),
	O_SHIPPRIORITY	integer,
	O_COMMENT	varchar(79)
);

alter table ORDERS
	add constraint ORDERS_PK primary key (O_ORDERKEY);

create table LINEITEM (
	L_ORDERKEY	integer not null,
	L_PARTKEY	integer not null,
	L_SUPPKEY	integer not null,
	L_LINENUMBER	integer not null,
	L_QUANTITY	numeric(18,2),
	L_EXTENDEDPRICE	numeric(18,2),
	L_DISCOUNT	numeric(18,2),
	L_TAX		numeric(18,2),
	L_RETURNFLAG	char(1),
	L_LINESTATUS	char(1),
	L_SHIPDATE	date,
	L_COMMITDATE	date,
	L_RECEIPTDATE	date,
	L_SHIPINSTRUCT	char(25),
	L_SHIPMODE	char(10),
	L_COMMENT	varchar(44)
);

alter table LINEITEM
	add constraint LINEITEM_PK primary key (L_ORDERKEY, L_LINENUMBER);

create table NATION (
	N_NATIONKEY	integer not null,
	N_NAME		char(25),
	N_REGIONKEY	integer not null,
	N_COMMENT	varchar(125)
);

alter table NATION
	add constraint NATION_PK primary key (N_NATIONKEY);

create table REGION (
	R_REGIONKEY	integer not null,
	R_NAME		char(25),
	R_COMMENT	varchar(152)
);

alter table REGION
	add constraint REGION_PK primary key (R_REGIONKEY);
!
isql -i $TMPSQL
if [ $? -ne 0 ]; then
	echo "Failed."
	exit 1
fi
echo "Done"

gfix -buffers $BUFFERS $PREFIX/tpcr.gdb

echo "Building the fill utility ..."
cd dbgen
if [ -e tpcr.gdb ]; then rm -f tpcr.gdb; fi
ln -s $PREFIX/tpcr.gdb tpcr.gdb
make
if [ $? -ne 0 ]; then
	echo "Failed."
	exit 1
fi
cd ..
echo "Done."

echo "Filling the database ..."
cd dbgen
./dbgen -s $SCALE -v -D -n $PREFIX/tpcr.gdb
if [ $? -ne 0 ]; then
	echo "Failed."
	exit 1
fi
cd ..
echo "Done."

echo "Doing a backup and restore to greatly improve the performance ..."
if [ $QUIET = "N" ]; then
	echo "`date`: backup to $PREFIX/tpcr.gbk started ..."
	gbak -b -v $PREFIX/tpcr.gdb $PREFIX/tpcr.gbk
else
	gbak -b $PREFIX/tpcr.gdb $PREFIX/tpcr.gbk
fi
if [ $? -ne 0 ]; then
	echo "Backup failed."
	exit 1
fi
if [ $QUIET = "N" ]; then
	echo "`date`: restore started ..."
	gbak -r -o -v $PREFIX/tpcr.gbk $PREFIX/tpcr.gdb
else
	gbak -r -o $PREFIX/tpcr.gbk $PREFIX/tpcr.gdb
fi
if [ $? -ne 0 ]; then
	echo "Restore failed."
	exit 1
fi
rm $PREFIX/tpcr.gbk
echo "Done."

echo "Creating the indexes, this may take a while ..."
cat >$TMPSQL <<!
connect '$PREFIX/tpcr.gdb';
create index lineitem_shipdate on lineitem(l_shipdate);
create index lineitem_partkey_suppkey on lineitem(l_partkey, l_suppkey);
create index part_brand_container_size on part(p_brand, p_container, p_size);
create index lineitem_quantity_sm_si on lineitem(l_quantity, l_shipmode, l_shipinstruct);
create index lineitem_shipmode_rd on lineitem(l_shipmode, l_receiptdate);
create index part_name on part(p_name);
create index supplier_nationkey on supplier(s_nationkey);
create index partsupp_suppkey on partsupp(ps_suppkey);
create index customer_nationkey on customer(c_nationkey);
create index orders_custkey on orders(o_custkey);
create index orders_orderdate on orders(o_orderdate);
!
isql -e -i $TMPSQL
if [ $? -ne 0 ]; then
	echo "Failed."
	exit 1
fi
echo "Done"
rm $TMPSQL

